/*
 * @(#)DrawLogo.java
 *
 */

import java.awt.*;
import java.awt.Graphics;
import java.awt.image.ImageProducer;
import java.applet.Applet;
import java.applet.AudioClip;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.net.URL;
import java.net.MalformedURLException;

/**
 * This applet draws the prologic logo and then undraws it.
 */

public class DrawLogo extends Applet implements Runnable {

    Point pnt[] = new Point[9];
    Point P3 = new Point(0,0);

/* MediaTracker 'class' ID numbers.  */

    static final int BACKGROUND_ID = 1;
    static final int ANIMATION_ID  = 2;
    static final int drawit = 1;
    static final int numsegs =20;
    static final int segPercent = 5;

    static final int box_x = 15, box_y = 3, box_w = 101, box_h = 98;
    static final int ytop = box_y+12, ybtm_t = box_y+60, ybtm_m = box_y+70, ybtm_b = box_y+80;
    static final int xmid_l = box_x+39, xmid_m = box_x+50, xmid_r = box_x+61;
    static final int xlft_l = box_x+10, xlft_m = box_x+15, xlft_r = box_x+20;
    static final int xrt_l = box_x+80, xrt_m = box_x+85, xrt_r = box_x+90;

    static final int dx1=3, dx2=3, dy=5, dx30=2, dx60=1;

    URL backgroundImageURL = null;

    Image backgroundImage = null;

//    Color backgroundColor = getBackground();
    Color backgroundColor = Color.white;  // new Color(0x006060);

    URL hrefURL = null;  // URL to link to, if any.

    int appWidth = 0;
    int appHeight = 0;

    boolean imageLoadError = false;

    URL imageSource = null;  // URL from which the images are loaded
    Thread engine = null;    // thread drawing the logo.

    int frameNum;   // current loop slot - index of line being drawn
    int part;
    int frameInc;
    int endImage = 10;

    int xPos = 0;  // current X position (for painting).
    int yPos = 0;  // current Y position (for painting).

    public static final int defaultPause = 500;  // milliseconds
    int globalPause = defaultPause;   // actual default
    int pauseStart = defaultPause;
    int pauseEnd = defaultPause;
    boolean userPause = false;

    boolean repeat;  // Repeat the animation?  If false, just play it once.
    boolean undraw;  // continue by reversing?

    Image offScrImage;     // The offscreen image, used in double buffering
    Graphics offScrGC;     // The offscreen graphics context, used in double buffering
    MediaTracker tracker;  // The MediaTracker we use to load our images.

    boolean loaded = false;  // Can we paint yet?

    boolean error = false;  // Was there an initialization error?

    /*  What we call an image file in messages.  */
    static final String imageLabel = "image";

    static final boolean debug = false;  // Print debugging info?

    DescriptionFrame description = null;

    public String getAppletInfo() {
        return "DrawLogo - Prologic logo animator";
    }

    public String[][] getParameterInfo() {
        String[][] info = {
            {"imagesource",     "URL",          "a directory"},
            {"backgroundcolor", "int",          "(24-bit number) displayed as background"},
            {"background",      "URL",          "image displayed as background"},
            {"endimage",        "int",          "end index"},
            {"namepattern",     "URL",          "used to generate indexed names"},
            {"href",            "URL",          "page to visit on mouse-click"},
            {"pause",           "int",          "milliseconds"},
            {"pausestart",      "int",          "milliseconds"},
            {"pauseend",        "int",          "milliseconds"},
            {"repeat",          "boolean",      "repeat or not"},
            {"undraw",          "boolean",      "reverse or not"},
        };
        return info;
    }

    /**
     * Show a crude "About" box.
     */
    void showDescription() {
        if (description == null) {
            description = new DescriptionFrame();
            String params[][] = getParameterInfo();

            description.tell("\t\t"+getAppletInfo()+"\n");
            description.tell("Document base: "+getDocumentBase()+"\n");
            description.tell("Code base: "+getCodeBase()+"\n\n");
            description.tell("Applet parameters:\n\n");

            description.tell("width = "+super.getParameter("WIDTH")+"\n");
            description.tell("height = "+super.getParameter("HEIGHT")+"\n");

            for (int i = 0; i < params.length; i++) {
                String name = params[i][0];
                description.tell(name+" = "+super.getParameter(name)+"\n");
            }
        }
        description.show();
    }

    /*  Print debugging info to the standard output.  */
    void dbg(String s) {
        if (debug) {
            System.out.println("> "+s);
        }
    }

    /**
     * Local version of getParameter for debugging purposes.
     */
    public String getParameter(String key) {
        String result = super.getParameter(key);
        dbg("getParameter("+key+") = "+result);
        return result;
    }

    /**
     * Parse a String of form xxx@yyy and return a Point.
     */
    Point parsePoint(String s) throws ParseException {
        int atPos = s.indexOf('@');
        if (atPos == -1) throw new ParseException("Illegal position: "+s);
        return new Point(Integer.parseInt(s.substring(0, atPos)),
                         Integer.parseInt(s.substring(atPos + 1)));
    }

    /**
     * Get the dimensions of an image.
     * @return the image's dimensions.
     */
    Dimension getImageDimensions(Image im) {
        return new Dimension(im.getWidth(null), im.getHeight(null));
    }

    /**
     * Initialize the applet.  Get parameters.
     */
    public void init() {

        tracker = new MediaTracker(this);
        setBackground(backgroundColor);

        appWidth = size().width;
        appHeight = size().height;
        setPoints();

        try {
            String param = getParameter("BACKGROUNDCOLOR");
            if (param != null) {
                backgroundColor = decodeColor(param);
            }
            repaint();
            param = getParameter("IMAGESOURCE");
            imageSource = (param == null) ? getDocumentBase() : new URL(getDocumentBase(), param + "/");

            String href = getParameter("HREF");
            if (href != null) {
                try {
                    hrefURL = new URL(getDocumentBase(), href);
                } catch (MalformedURLException e) {
                    showParseError(e);
                }
            }
            param = getParameter("PAUSE");
            globalPause =
                (param != null) ? Integer.parseInt(param) : defaultPause;
            param = getParameter("REPEAT");
            repeat = (param == null) ? true : (param.equalsIgnoreCase("yes") ||
                                               param.equalsIgnoreCase("true"));
            param = getParameter("UNDRAW");
            undraw = (param == null) ? true : (param.equalsIgnoreCase("yes") ||
                                               param.equalsIgnoreCase("true"));
            param = getParameter("ENDIMAGE");
            if (param != null) {
                endImage = Integer.parseInt(param);
            }

            param = getParameter("BACKGROUND");
            if (param != null) {
                backgroundImageURL = new URL(imageSource, param);
            }

            param = getParameter("PAUSE_START");
            pauseStart =
                (param != null) ? Integer.parseInt(param) : defaultPause;

            param = getParameter("PAUSE_END");
            pauseEnd =
                (param != null) ? Integer.parseInt(param) : defaultPause;

        } catch (MalformedURLException e) {
            showParseError(e);
        }

        frameNum = 0;
        part = 1;
    }

    Color decodeColor(String s) {
        int val = 0;
        try {
            if (s.startsWith("0x")) {
                val = Integer.parseInt(s.substring(2), 16);
            } else if (s.startsWith("#")) {
                val = Integer.parseInt(s.substring(1), 16);
            } else if (s.startsWith("0") && s.length() > 1) {
                val = Integer.parseInt(s.substring(1), 8);
            } else {
                val = Integer.parseInt(s, 10);
            }
            return new Color(val);
        } catch (NumberFormatException e) {
            return null;
        }
    }

    void tellLoadingMsg(String file, String fileType) {
        showStatus("Animate: loading "+fileType+" "+file);
    }

    void tellLoadingMsg(URL url, String fileType) {
        tellLoadingMsg(url.toExternalForm(), fileType);
    }

    void drawBackgroundBox(Graphics g) {
        Color bgc = new Color(0x006060);
        g.setColor(bgc);
        g.fillRect(box_x, box_y, box_w, box_h);
    }

    void drawLoadingMsg(Graphics g) {
        drawBackgroundBox(g);
        Font f = new Font("Helvetica", Font.BOLD, 12);
        g.setColor(Color.white);
        g.setFont(f);
        g.drawString("Loading",25,50);
        g.drawString("image...",25,65);
//        g.drawString("please wait.",25,80);
    }

    void drawFatLine(Graphics g, Point p1, Point p2, int percent) {
        int xpoints[] = new int[4];
        int ypoints[] = new int[4];
        Point p3 = new Point(p2.x, p2.y);
        if (percent == 0) return;
        if (percent <= 99) {
            int q = 100 - percent;
            int xp3 = (q*p1.x + percent*p2.x)/100;
            int yp3 = (q*p1.y + percent*p2.y)/100;
            p3.x = xp3;
            p3.y = yp3;
        }
        if (p1.y < p3.y) {
            ypoints[0] = p1.y;
            ypoints[1] = p1.y;
            ypoints[2] = p3.y;
            ypoints[3] = p3.y;
            xpoints[0] = p1.x - dx1;
            xpoints[1] = p1.x + dx2;
            xpoints[2] = p3.x + dx2;
            xpoints[3] = p3.x - dx1;
        }
        else if (p1.y > p3.y) {
            ypoints[0] = p1.y;
            ypoints[1] = p1.y;       //  + dy
            ypoints[2] = p3.y;
            ypoints[3] = p3.y;
            xpoints[0] = p1.x - dx1;
            xpoints[1] = p1.x + dx2;
            xpoints[2] = p3.x + dx2;
            xpoints[3] = p3.x - dx1;
        }
        else {                             // horizontal line
            ypoints[0] = p1.y;
            ypoints[1] = p1.y + dy;
            ypoints[2] = ypoints[1];
            ypoints[3] = ypoints[0];
            xpoints[0] = p1.x - dx60;  //  0______3
            xpoints[1] = p1.x + dx30;  //   \____/
            xpoints[2] = p3.x - dx30;  //   1    2
            xpoints[3] = p3.x + dx60;
        }
        g.fillPolygon(xpoints,ypoints,4);
    }

    void setPoints() {
        pnt[0] = new Point(xlft_m,ybtm_m);
        pnt[1] = new Point(xrt_m,ybtm_m);
        pnt[2] = new Point(xmid_m, ytop);
        pnt[3] = new Point(xlft_l, ybtm_t);
        pnt[4] = new Point(xrt_r,ybtm_t);
        pnt[5] = new Point(xmid_r, ytop);
        pnt[6] = new Point(xlft_r, ybtm_b);
        pnt[7] = new Point(xrt_l, ybtm_b);
        pnt[8] = new Point(xmid_l, ytop);
    }

    void drawLogoLine(Graphics g, int lineno, int percent) {
        drawBackgroundBox(g);
        g.setColor(Color.white);
        switch (lineno) {
        case 9:
            drawFatLine(g, pnt[8], pnt[3], percent);
            percent = 100;
        case 8:
            drawFatLine(g, pnt[7], pnt[8], percent);
            percent = 100;
        case 7:
            drawFatLine(g, pnt[6], pnt[7], percent);
            percent = 100;
        case 6:
            drawFatLine(g, pnt[5], pnt[6], percent);
            percent = 100;
        case 5:
            drawFatLine(g, pnt[4], pnt[5], percent);
            percent = 100;
        case 4:
            drawFatLine(g, pnt[3], pnt[4], percent);
            percent = 100;
        case 3:
            drawFatLine(g, pnt[2], pnt[0], percent);
            percent = 100;
        case 2:
            drawFatLine(g, pnt[1], pnt[2], percent);
            percent = 100;
        case 1:
            drawFatLine(g, pnt[0], pnt[1], percent);
        case 0:
            break;
        }
    }

    void clearLoadingMessage() {
        showStatus("");
    }

    void loadError(String fileName, String fileType) {
        String errorMsg = "Animate: Couldn't load "+fileType+" "+
            fileName;
        showStatus(errorMsg);
        System.err.println(errorMsg);
        error = true;
        repaint();
    }

    void loadError(URL badURL, String fileType) {
        loadError(badURL.toExternalForm(), fileType);
    }

    void showParseError(Exception e) {
        String errorMsg = "Animate: Parse error: "+e;
        showStatus(errorMsg);
        System.err.println(errorMsg);
        error = true;
        repaint();
    }

    /**
     * Run the animation. This method is called by class Thread.
     * @see java.lang.Thread
     */
    public void run() {
        Thread me = Thread.currentThread();
        URL badURL;

        me.setPriority(Thread.MIN_PRIORITY);

        if (! loaded) {
            try {
                // ... to do a bunch of loading.

                if (backgroundImageURL != null) {
                    tellLoadingMsg(backgroundImageURL, imageLabel);
                    backgroundImage = getImage(backgroundImageURL);
                    tracker.addImage(backgroundImage, BACKGROUND_ID);
                    tracker.waitForID(BACKGROUND_ID);
                    if (tracker.isErrorID(BACKGROUND_ID)) {
                        loadError(backgroundImageURL, "background image");
                    }
                    repaint();
                }

                // Fetch the animation frames

                clearLoadingMessage();

                offScrImage = createImage(appWidth, appHeight);
                offScrGC = offScrImage.getGraphics();
                offScrGC.setColor(Color.lightGray);

                loaded = true;
                error = false;
            } catch (Exception e) {
                error = true;
                e.printStackTrace();
            }
        }

        if (userPause) {
            return;
        }

        try {
            if (endImage > 1) {
                frameInc = 1;
                while (appWidth > 0 && appHeight > 0 && engine == me) {
                    if (frameNum >= endImage) {
                        if (!repeat) {
                            return;
                        }
                        if (!undraw) {
                            frameNum = 0;
                        } else {
                            frameNum -= 1;
                            frameInc = -1;
                            part = numsegs;
                        }
                    } else if (frameNum < 0) {
                        frameNum = 0;
                        part = 0;
                        frameInc = 1;
                    }
                    repaint();

                    try {
                        int pause = globalPause;
                        if (frameNum == endImage-1 && part ==numsegs)
                                pause = pauseEnd;
                        if (frameNum == 0 && part ==0) pause = pauseStart;
                        Thread.sleep(pause);
                    } catch (InterruptedException e) {
                        // Should we do anything?
                    }
                    part = part + frameInc;
                    if (part < 0 || part > numsegs) {
                        frameNum = frameNum+frameInc;
                        if (part <= 0) part = numsegs;
                        else part = 0;
                    }
                }
            }
        } finally {
            ; // do nothing
        }
    }

    /* No need to clear anything; just paint.  */
    public void update(Graphics g) {
        paint(g);
    }

    /* Paint the current frame.  */
    public void paint(Graphics g) {
        if (error || !loaded) {
            g.clearRect(0, 0, appWidth, appHeight);
            drawLoadingMsg(g);
        } else {
            if (backgroundImage != null) {
                    offScrGC.drawImage(backgroundImage, xPos, yPos, this);
            }
            drawLogoLine(offScrGC, frameNum, part * segPercent);
            g.drawImage(offScrImage, 0, 0, this);
        }
    }

    public void start() {   // Start the applet by forking an animation thread.
        if (engine == null) {
            engine = new Thread(this);
            engine.start();
        }
        showStatus(getAppletInfo());
    }

    public void stop() {
        if (engine != null && engine.isAlive()) {
            engine.stop();
        }
        engine = null;
    }

    /* Pause the thread when the user clicks the mouse in the applet.
       If the thread has stopped restart it.     */
    public boolean handleEvent(Event evt) {
        switch (evt.id) {
        case Event.MOUSE_DOWN:
            if ((evt.modifiers & Event.SHIFT_MASK) != 0) {
                showDescription();
                return true;
            } else if (hrefURL != null) {
                // let mouse-up handle going to the new page
                return true;
            } else if (loaded) {
                if (engine != null && engine.isAlive()) {
                    if (userPause) {
                        engine.resume();
                    } else {
                        engine.suspend();
                    }
                    userPause = !userPause;
                } else {
                    userPause = false;
                    frameNum = 0;
                    engine = new Thread(this);
                    engine.start();
                }
            }
            return true;
        case Event.MOUSE_UP:
            if (hrefURL != null && ((evt.modifiers & Event.SHIFT_MASK) == 0)) {
                getAppletContext().showDocument(hrefURL);
            }
            return true;
        case Event.MOUSE_ENTER:
            showStatus(getAppletInfo()+" -- shift-click for info");
            return true;
        case Event.MOUSE_EXIT:
            showStatus("");
            return true;
        case Event.KEY_ACTION:
        case Event.KEY_RELEASE:
        case Event.KEY_ACTION_RELEASE:
            dbg("Got event "+evt);
            return true;
        default:
            return super.handleEvent(evt);
        }
    }

}


class ParseException extends Exception {
    ParseException(String s) {
        super(s);
    }
}


class DescriptionFrame extends Frame {

    static final int rows = 26;
    static final int cols = 70;

    TextArea info;

    DescriptionFrame() {
        super("Animate");
        add("Center", info = new TextArea(rows, cols));
        info.setEditable(false);
        info.setBackground(Color.white);
        Panel buttons = new Panel();
        buttons.setLayout(new FlowLayout());
        add("South", buttons);
        buttons.add(new Button("Cancel"));

        pack();
    }

    public void show() {
        info.select(0,0);
        super.show();
    }

    void tell(String s) {
        info.appendText(s);
    }

    public boolean action(Event e, Object arg) {
        if (e.target instanceof Button) {
            hide();
            return true;
        }
        return false;
    }
}

